#include <bdd.h>
#include <pa.h>
#include <versis.h>
#include <mc.h>

#define PROGNAME "demoAutomata"

void hello() {
  printf("Witness and Counterexample Automata Generator\n");
  printf("Online on http://fmt.isti.cnr.it:8080/WCA/\n");
  printf("This program is a part of EST 2nd Edition (version 2.3beta)\n");
}

int main(int argc, char **argv)
{
  int i;
  int mctype;
  Bdd_Boolean help;
  Bdd_String ccsname,actlname,procname;
  int p;

  mctype = MC_AUTOMATON;
  ccsname = NULL;
  actlname = NULL;
  procname = strdup("S");

  if (argc == 0) {
    help = TRUE;
  } else {
    help = FALSE;
  }

  i = 1;
  while (i<argc) {

    if (!strcmp(argv[i],"-help")) {
      help = TRUE;
    }

    if (!strcmp(argv[i],"-explain") || !strcmp(argv[i],"-e")) {
      mctype =  mctype + MC_EXPLAIN;
    }

    if (!strcmp(argv[i],"-model") || !strcmp(argv[i],"-m")) {
      ccsname = strdup(argv[++i]);
    }

    if (!strcmp(argv[i],"-logic") || !strcmp(argv[i],"-l")) {
      actlname = strdup(argv[++i]);
    }

    i++;
  }

  hello();

  if (help) {
    printf("Usage: %s [-explain] -model ccsname -logic actlname\n",PROGNAME);
    printf("Parameters:\n");
    printf("  -explain         trace the algorithm for automata generation\n");
    printf("  -model ccsname   the file with CCS model description\n");
    printf("  -logic actlname  the file with actl formulae\n");
    return 1;
  }

  if (!ccsname) {
    printf("\nERROR: The file with CCS model description must be specified.\n");
    printf("HELP: Use: -model ccsname\n");
    return 1;
  }

  if (!actlname) {
    printf("\nERROR: The file with actl formulae must be specified.\n");
    printf("HELP: Use: -logic ccsname\n");
    return 1;
  }

  /* *********************************************************************** */
  /* *********************************************************************** */

  Bdd_InitPkg();
  Pa_InitPkg();
  Versis_InitPkg();
  Mc_InitPkg();
 
  /* *********************************************************************** */
  printf("\nLoading model");
  /* *********************************************************************** */

  Versis_ParseCCS(ccsname);

  p = Pa_FindProcess(procname);
  if (p == -1) {
    printf("\nProcess %s does not exist.\n",procname);
  } else {

    Pa_EncodeProcess(procname);

    printf("\nProcess %s has %d transitions.\n",procname,
                                           pa_processTable[p].numTransitions);

    /* *********************************************************************** */
    printf("\n==============================================\n");
    printf("Model checking and a generation of w/c automaton");
    printf("\n==============================================\n\n");
    /* *********************************************************************** */

    Mc_CheckACTL(0,procname,actlname,mctype);
  }


  return(0);
}
