/* *****************************************************************

Parameters:
-help
-model
-process

-trace = trace minimization
-weak = weak minimization
-strong = strong minimization

****************************************************************** */

#include <bdd.h>
#include <pa.h>
#include <versis.h>

#define PROGNAME "min"

static void concat(char **s1, const char *s2);

void hello() {
  printf("Minimization\n");
  printf("This program is a part of EST 2nd Edition (version 2.3)\n");
}

int main(int argc, char **argv)
{
  int i;
  int mctype;
  Bdd_Boolean help;
  int type;
  Bdd_String ccsname,procname,outname;
  int p;

  ccsname = NULL;
  procname = NULL;

  if (argc == 1) {
    help = TRUE;
  } else {
    help = FALSE;
  }

  type = -1;

  i = 1;
  while (i<argc) {

    if (!strcmp(argv[i],"-help")) {
      help = TRUE;
    }

    if (!strcmp(argv[i],"-model")) {
      ccsname = strdup(argv[++i]);
    }

    if (!strcmp(argv[i],"-process")) {
      procname = strdup(argv[++i]);
    }

    if (!strcmp(argv[i],"-strong")) {
      if (type == -1) {
  	outname = strdup("STRONG_");
        type = 0;
      } else {
        help = TRUE;
      }
    }

    if (!strcmp(argv[i],"-weak")) {
      if (type == -1) {
  	outname = strdup("WEAK_");
        type = 1;
      } else {
        help = TRUE;
      }
    }

    if (!strcmp(argv[i],"-trace")) {
      if (type == -1) {
  	outname = strdup("TRACE_");
        type = 3;
      } else {
        help = TRUE;
      }
    }

    i++;
  }

  /*
  hello();
  */

  if (type == -1) {
    help = TRUE;
  }

  if (help) {
    printf("Usage: %s -trace -model ccsname -process procname\n",PROGNAME);
    printf("Parameters:\n");
    printf("  -model ccsname    the file with CCS model description\n");
    printf("  -process procname the name of process to be minimized\n");
    printf("Morevover, you must add one of the following parameters:\n");
    printf("  -trace = trace minimization\n");
    printf("  -weak = weak minimization\n");
    printf("  -strong = strong minimization\n");
    return 1;
  }

  if (!ccsname) {
    printf("\nERROR: The file with CCS model description must be specified.\n");
    printf("HELP: Use: -model ccsname\n");
    return 1;
  }

  if (!procname) {
    printf("\nERROR: The name of the process must be specified.\n");
    printf("HELP: Use: -process procname\n");
    return 1;
  }

  /* *********************************************************************** */
  /* *********************************************************************** */

  Bdd_InitPkg();
  Pa_InitPkg();
  Versis_InitPkg();
 
  /* *********************************************************************** */
  printf("Loading model");
  /* *********************************************************************** */

  Versis_ParseCCS(ccsname);

  p = Pa_FindProcess(procname);
  if (p == -1) {
    printf("\nProcess %s does not exist.\n",procname);
  } else {

    printf("\n  Process %s has %d transitions.\n\n",procname,
                                           pa_processTable[p].numTransitions);

    /* *********************************************************************** */
    printf("Result\n");
    /* *********************************************************************** */

    Pa_EncodeProcess(procname);

    Versis_Minimization(type,0,procname);

    concat(&outname,procname);
    Pa_WriteProcessCCS(outname,0);
  }

  return(0);
}

/* ******************************************************* */
/* LOCAL FUNCTIONS - it is a good idea to make them static */
/* ******************************************************* */

static void
concat(char **s1, const char *s2)
{
   *s1 = (char*) realloc(*s1,strlen(*s1)+strlen(s2)+1);
   strcat(*s1,s2);
}
