/* *****************************************************************

Parameters:
-l or -logic
-m or -model

Default is model checking only.
-p or -path = linear witness/counterexample
-a or -automaton = witness/counterexample automaton
-e or -explain = explain linear w/c and w/c automaton

****************************************************************** */

#include <bdd.h>
#include <pa.h>
#include <versis.h>
#include <mc.h>

#define PROGNAME "wca"

void hello() {
  printf("=======================================================================\n");
  printf("   W i t n e s s   a n d   C o u n t e r e x a m p l e   S e r v e r   \n");
  printf("        Witness and Counterexample Automata Generator for ACTL         \n");
  printf("          (based on EST 2nd Edition - ver 2.4a - 25.01.2005)           \n");
  printf("=======================================================================\n");
}

int main(int argc, char **argv)
{
  int i;
  int mctype;
  Bdd_Boolean help,path,automaton,explain;
  Bdd_String ccsname,actlname,procname;
  int p;

  mctype = MC_SIMPLE;
  ccsname = NULL;
  actlname = NULL;
  procname = strdup("S");

  if (argc == 0) {
    help = TRUE;
  } else {
    help = FALSE;
  }

  path = FALSE;
  automaton = FALSE;
  explain = FALSE;

  i = 1;
  while (i<argc) {

    if (!strcmp(argv[i],"-help")) {
      help = TRUE;
    }

    if (!strcmp(argv[i],"-model") || !strcmp(argv[i],"-m")) {
      ccsname = strdup(argv[++i]);
    }

    if (!strcmp(argv[i],"-logic") || !strcmp(argv[i],"-l")) {
      actlname = strdup(argv[++i]);
    }

    if (!strcmp(argv[i],"-path") || !strcmp(argv[i],"-p")) {
      path = TRUE;
      mctype =  mctype | MC_DIAGNOSTIC | MC_TRACEPATH;
    }

    if (!strcmp(argv[i],"-automaton") || !strcmp(argv[i],"-a")) {
      automaton = TRUE;
      mctype =  mctype | MC_AUTOMATON | MC_MINIMIZE;
    }

    if (!strcmp(argv[i],"-explain") || !strcmp(argv[i],"-e")) {
      explain = TRUE;
      mctype =  mctype | MC_EXPLAIN;
    }

    i++;
  }

  hello();

  if (help) {
    printf("Usage: %s [-explain] -model ccsname -logic actlname\n",PROGNAME);
    printf("Parameters:\n");
    printf("  -path            generate a linear witness or a linear counterexample\n");
    printf("  -automaton       generate witness or counterexample automaton\n");
    printf("  -explain         trace the algorithm\n");
    printf("  -model ccsname   the file with CCS model description\n");
    printf("  -logic actlname  the file with actl formulae\n");
    return 1;
  }

  if (!ccsname) {
    printf("\nERROR: The file with CCS model description must be specified.\n");
    printf("HELP: Use: -model ccsname\n\n");
    return 1;
  }

  if (!actlname) {
    printf("\nERROR: The file with actl formulae must be specified.\n");
    printf("HELP: Use: -logic ccsname\n\n");
    return 1;
  }

  /* *********************************************************************** */
  /* *********************************************************************** */

  Bdd_InitPkg();
  Pa_InitPkg();
  Versis_InitPkg();
  Mc_InitPkg();
 
  /* *********************************************************************** */
  /* *********************************************************************** */

  printf("\nLoading...");
  Versis_ParseCCS(ccsname);

  p = Pa_FindProcess(procname);
  if (p == -1) {
    printf("\nProcess %s does not exist.\n",procname);
  } else {

    printf("\n");
    printf("-----------------\n");
    printf("Report on model S");
    printf("\n-----------------\n\n");

    if (explain) {
      Pa_WriteProcessCCS(procname,1);
    } else {
      Pa_WriteProcess(procname,0);
    }

    /* *********************************************************************** */
    printf("------------------------------\n");
    printf("ACTL model checking on model S");
    printf("\n------------------------------\n\n");
    /* *********************************************************************** */

    Mc_CheckACTL(0,procname,actlname,mctype);
  }

  return(0);
}
