/**CFile*******************************************************************
  PackageName [bdd]
  FileName    [bddOutput.c]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Ales Casar (casar@uni-mb.si)]

  Revision    [$Revision: 2.3 $]
  Date        [$Date: 2004/07/22 14:32:04 $]
  Synopsis    [Package 'bdd' provides data structures and algorithms
               for the representation and manipulation of Boolean
               functions with ROBDDs. A hash table is used for quick
               search of nodes. Complement edges decreases the number
               of nodes. Function ITE is given for manipulation of
               boolean functions. An automatic garbage collection with
               a formulae counter is implemented.]
  Description [The file bddOutput.c contains functions for writing and
               drawing logical functions and BDDs.]
  SeeAlso     [bdd.h, bddInt.h]
  Copyright   [Copyright (C) 2000-2004 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#include "bddInt.h"

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

typedef struct SezBesed {
  struct SezBesed *p;
  Bdd_String ime;
  Bdd_Boolean mark;
} SezBesed;

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static void WriteList(SezBesed *s);

static void WriteFunction_R(Bdd_Node *f, Bdd_Boolean pik, SezBesed *ze,
                            int *large);

static void WriteBDD_R(Bdd_Edge f, int *large);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Bdd_WriteFunction.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Bdd_WriteFunction(Bdd_Edge f)
{
  int large;

  large = 100;
  WriteFunction_R(f.p, f.mark, NULL, &large);
  printf("\n");
}

/**Function****************************************************************
  Synopsis    [Function Bdd_WriteBDD.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Bdd_WriteBDD(Bdd_Edge f)
{
  int large;

  large = 2000;
  WriteBDD_R(f, &large);
  printf("\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function WriteList.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
WriteList(SezBesed *s)
{
  if (s) {
    WriteList(s->p);
    printf(" ");
    if (s->mark) printf("*");
    printf("%s",s->ime);
  }
}

/**Function****************************************************************
  Synopsis    [Function WriteFunction_R.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
WriteFunction_R(Bdd_Node *f, Bdd_Boolean pik, SezBesed *ze, int *large)
{
  SezBesed spr;

  if (*large == 0) return;

  if (f == bdd_termTrue.p) {
    if (!pik) {
      printf("  + ");
      if (ze) {
	WriteList(ze);
	printf("\n");
        if (--(*large) == 0) {
          printf("Function to large.");
          return;
        }
      }
      else
        printf(" 1");
    }
    else
      if (!ze)
	printf("  +  0");
  } else {
    spr.p = ze;
    spr.ime = f->v->name;
    spr.mark = TRUE;
    WriteFunction_R(f->f.p, pik ^ f->f.mark, &spr, large);
    spr.mark = FALSE;
    WriteFunction_R(f->t.p, pik ^ f->t.mark, &spr, large);
  }
}

/**Function****************************************************************
  Synopsis    [Function WriteBDD_R.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
WriteBDD_R(Bdd_Edge f, int *large)
{

  if (*large == 0) return;
  if (--(*large) == 0) {
    printf("\nBDD to large.");
    return;
  }

  if (f.mark) {
    printf("* ");
  }
  printf("%s",f.p->v->name);
  if (f.p != bdd_termTrue.p) {
    printf(" (");
    WriteBDD_R(f.p->f,large);
    if (*large) printf(") (");
    WriteBDD_R(f.p->t,large);
    if (*large) printf(")");
  }
}
