/**CFile*******************************************************************
  PackageName [mc]
  FileName    [mcTcl.c]
  Author      [Robert Meolic (meolic@uni-mb.si)]

  Revision    [$Revision: 2.15 $]
  Date        [$Date: 2005/01/25 10:16:14 $]
  Synopsis    [Package 'mc' provides tools for ACTL model checking]
  Description [The file mcTcl.c contains definitions of Tcl commands,
               which can be used for manipulating from Tcl interpreter
               (e.g. tclsh or wish).]
  SeeAlso     [mc.h, mcInt.h]
  Copyright   [Copyright (C) 2000-2004 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#include "mcInt.h"

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
#define USECONST const

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int McInitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                        int argc, USECONST char **argv);

static int McAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                         int argc, USECONST char **argv);

static int McCheckACTLCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Mc_Init.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Mc_Init(Tcl_Interp *interp)
{
  Tcl_LinkVar(interp, "mc_simple", (char *) &MC_SIMPLE,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_trace", (char *) &MC_TRACE,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_xtrace", (char *) &MC_XTRACE,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_explain", (char *) &MC_EXPLAIN,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_tracepath", (char *) &MC_TRACEPATH,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_diagnostic", (char *) &MC_DIAGNOSTIC,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_automaton", (char *) &MC_AUTOMATON,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_minimize", (char *) &MC_MINIMIZE,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_LinkVar(interp, "mc_tree", (char *) &MC_TREE,
                     TCL_LINK_INT || TCL_LINK_READ_ONLY);

  Tcl_CreateCommand(interp, "mc_initPkg", McInitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "mc_aboutPkg", McAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "mc_check_actl", McCheckACTLCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  return Tcl_PkgProvide(interp, "mc", "1.0");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function McInitCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
McInitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                  USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  printf("Initialization of Model checking package... ");
  Mc_InitPkg();
  printf("OK");

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function McAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
McAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  Mc_AboutPkg();

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function McCheckACTLCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
McCheckACTLCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Bdd_String s1,s2,s3,s4;
  int par;
  int parserSettings;

  if ((argc < 4) && (argc > 5)) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  sscanf(s1,"%d",&par);
  free(s1);

  if ((par != 0) && (par != 1)) {
    interp->result = "first arg is wrong";
    return TCL_ERROR;
  }

  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);

  if (argc == 5) {
    s4 = strdup(argv[4]);
    if (!strcmp(s4,"simple")) parserSettings = MC_SIMPLE;
    else if (!strcmp(s4,"diagnostic")) parserSettings = MC_DIAGNOSTIC;
    else if (!strcmp(s4,"automaton")) parserSettings = MC_AUTOMATON | MC_MINIMIZE;
    else parserSettings = atoi(s4);
    free(s4);
  } else {
    parserSettings = 0;
  }

  printf("ACTL model checking ");
  if (par == 0) printf("on process ");
  if (par == 1) printf("on composition ");
  printf(s2);
  printf("\n\n");

  Tcl_Eval(interp,"disable_input");
#ifdef USETHREADS
  Mc_CheckACTL_Thr(par,s2,s3,parserSettings);
#else
  Mc_CheckACTL(par,s2,s3,parserSettings);
#endif
  Tcl_Eval(interp,"enable_input");

  free(s2);
  free(s3);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}
