/**CFile*******************************************************************
  PackageName [strucval]
  FileName    [strucvalMain.h]
  Authors     [Robert Meolic (meolic@uni-mb.si), Marina Bagic (??)]

  Revision    [$Revision:$]
  Date        [$Date:$]
  Synopsis    [Structure-driven validation of hardware and software systems]
  Description [The file strucvalMain.h contains main functions.]
  SeeAlso     [strucval.h, strucvalInt.h]
  Copyright   [Copyright (C) 2005 UM-FERI
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#include "strucvalInt.h"

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

int strucval_status = 0;               /* initialization status */

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int wordNumber(Bdd_String s);

void Strucval_SyncProduct(Bdd_String name, Bdd_String lts, Bdd_String wca,
                          Bdd_String actions);

void Strucval_FinalStates(Pa_Composition *c);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Strucval_Init.]
  Description [Creates tcl commands.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

#ifndef TCLOUT
   void Strucval_Init(void) {}
#endif

/**Function****************************************************************
  Synopsis    [Function Strucval_InitPkg.]
  Description [Initializes the package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Strucval_InitPkg()
{
  if (strucval_status == 1) {
    printf("\nStrucval package is already initialized. ");
    return;
  } else {
    strucval_status = 1;
  }

}

/**Function****************************************************************
  Synopsis    [Function Strucval_ExitPkg.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Strucval_ExitPkg()
{
}

/**Function****************************************************************
  Synopsis    [Function Strucval_AboutPkg]
  Description [Strucval_AboutPkg reports release of Strucval package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Strucval_AboutPkg() {
  printf("Strucval package, release ");
  printf(STRUCVAL_RELEASE);
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int main(int argc, char **argv)
{
  Bdd_String fileLTS;
  Bdd_String inputLTS;
  Bdd_String fileWCA;
  Bdd_String inputWCA;

  int lts,wca;

  /* *********************************************************************** */
  /* *********************************************************************** */

  Strucval_AboutPkg();

  /* *********************************************************************** */
  /* *********************************************************************** */

  Bdd_InitPkg();
  Pa_InitPkg();
  Versis_InitPkg();
  Mc_InitPkg();
  Strucval_InitPkg();

  /* *********************************************************************** */
  /* *********************************************************************** */

  fileLTS = strdup("test");
  inputLTS = strdup("LTS");
  inputWCA = strdup("WCA");

  printf("\nLoading...");
  Pa_ParseSortProcess(fileLTS);
  printf("\n");

  /* DEBUGGING */
  /*

  lts = Pa_FindProcess(inputLTS);
  if (lts == -1) {
    printf("\nProcess %s does not exist.\n",inputLTS);
  } else {
    Pa_WriteProcess(inputLTS,1);
  }
  wca = Pa_FindProcess(inputWCA);
  if (wca == -1) {
    printf("\nProcess %s does not exist.\n",inputWCA);
  } else {
    Pa_WriteProcess(inputWCA,1);
  }

  */

  /* *********************************************************************** */
  /* *********************************************************************** */

  free(fileLTS);
  free(inputLTS);
  free(inputWCA);

  printf("\nSTART OF PRODUCT CALCULATION\n");

  Pa_EncodeMultiProcesses("LTS WCA");
  Strucval_SyncProduct("PRODUCT","LTS","WCA","x");

  /* *********************************************************************** */
  /* *********************************************************************** */

  /*
  Pa_DecodeComposition("PRODUCT",TRUE);
  */
}

void
Strucval_SyncProduct(Bdd_String name, Bdd_String lts, Bdd_String wca, Bdd_String actions)
{
  static int iteration=0;

  int i,j,k,finalWCA;
  int numP,numA,numcomp,token;
  Bdd_String sup,sup_next,token1;
  Pa_Composition *c;
  Pa_Sort *s;
  Pa_Process *p;

  Bdd_Edge actionsBDD;
  Bdd_Edge s_new,s_acc;
  Bdd_Edge sup1,sup2;
  Bdd_Edge new1,new2,s1,s2,s3,s1_next,s2_next,s3_next;
  Bdd_Edge tr1,tr2,tau;
  Bdd_Edge finalBddWCA;

  numP = 2;
  numA = wordNumber(actions);

  /* CREATE EMPTY COMPOSITION c WITH NUMBER numcomp */
  /* TYPE OF COMPOSITION IS -1 */

  i = pa_compositions;
  numcomp = Pa_AddNewComposition(name,-1,2,numA);
  if (pa_compositions == i) {
    printf("Composition overwritten!\n");
  }
  c = &pa_compositionTable[numcomp];

  /* ADD INFORMATION ABOUT PROCESSES INTO c */

  i = Pa_FindProcess(lts);
 
  if (i == -1) {
    Pa_RemoveComposition(numcomp);
    printf(" LTS %s does not exist.\n",lts);
    return;
  }
  if (!pa_processTable[i].encoded) {
    Pa_RemoveComposition(numcomp);
    printf(" LTS %s is not encoded.\n",lts);
    return;
  }
  c->tableP[0] = i;                     /* number of first process */
  c->sort = pa_processTable[i].sort;

  i = Pa_FindProcess(wca);

  if (i == -1) {
    Pa_RemoveComposition(numcomp);
    printf(" WCA %s does not exist.\n",wca);
    return;
  }
  if (!pa_processTable[i].encoded) {
    Pa_RemoveComposition(numcomp);
    printf(" WCA %s is not encoded.\n",wca);
    return;
  }
  c->tableP[1] = i;
  if (c->sort != pa_processTable[i].sort) {
    Pa_RemoveComposition(numcomp);
    printf(" LTS and WCA have different sorts!\n");
    return;
  }

  /* THIS PART IS FOR DEBUGGING */
  /**/
  for(i=0; i<numP; i++) {
    printf("\nProcess number %d:\n",i);
    p = &pa_processTable[c->tableP[i]];
    Pa_WriteProcess(p->name,1);
    printf("\nBDD for its transition relation:\n");
    Bdd_WriteFunction(p->d);
  }
  /**/

  /* ADD INFORMATION ABOUT PROCESSES INTO c */

  /* ************************************** */
  /* THIS PART IS ABOUT STRING PARSING      */
  /* ************************************** */

  sup = (Bdd_String) strdup(actions);

  token = strcspn(sup," ");
  token1 = (Bdd_String) malloc(token+1);
  strncpy(token1,sup,token);            /* name of first action */
  token1[token] = '\0';

  i = Pa_FindSortAction(&pa_sortTable[c->sort],token1);
  free(token1);
 
  if (i == -1) {
    Pa_RemoveComposition(numcomp);
    printf(" Action %%%d does not exist.\n",1);
    return;
  }
  c->tableA[0] = i;  /* number of first action */

  k = 1;
  while (k < numA) {
    while (sup[token] == ' ') token = token + 1;
    sup_next = (Bdd_String) strdup(&sup[token]);
    free(sup);
    sup = sup_next;

    token = strcspn(sup," ");
    token1 = (Bdd_String) malloc(token+1);
    strncpy(token1,sup,token);           /* name of next action */
    token1[token] = '\0';

    i = Pa_FindSortAction(&pa_sortTable[c->sort],token1);
    free(token1);

    if (i == -1) {
      Pa_RemoveComposition(numcomp);
      printf(" Action %%%d does not exist.\n",k+1);
      return;
    }
    c->tableA[k++] = i;      /* sequence number of first action */
  }
  free(sup);

  /* ************************************** */
  /* END OF STRING PARSING                  */
  /* ************************************** */

  s = &pa_sortTable[c->sort];
  tau = pa_sortTable[c->sort].table[0].p;

  sup1 = bdd_termFalse;
  for (i=0; i<c->numActions; i++) {
    sup2 = Bdd_ITE(s->a0,s->table[c->tableA[i]].p,bdd_termFalse);
    sup1 = Bdd_ITE(sup1,bdd_termTrue,sup2);
    sup2 = Bdd_ITE(s->a0,bdd_termFalse,s->table[c->tableA[i]].p);
    sup1 = Bdd_ITE(sup1,bdd_termTrue,sup2);
  }
  actionsBDD = sup1;

  /* CALCULATE INITIAL STATE OF THE COMPOSITION */

  sup1 = bdd_termTrue;
  for(i=0; i<numP; i++) {
    p = &pa_processTable[c->tableP[i]];

    /*
    printf("\nBDD for initial state of process number %d:\n",i);
    Bdd_WriteFunction(p->tableS[p->initial].bddR);
    */

    sup1 = Bdd_ITE(sup1,p->tableS[p->initial].bddR,bdd_termFalse); /* AND */
  }
  c->initialBDD = sup1;

  /* THIS PART IS FOR DEBUGGING */
  /**/
  printf("\nBDD for initial state of the composition:\n");
  Bdd_WriteFunction(c->initialBDD);
  Pa_DecodeCompStates(c,c->initialBDD,TRUE);
  printf("\n");
  /**/

  /* s_new (RRR) CONTAINS NEW STATES FOUND IN PREVIOUSLY STEP OF CALCULATION */
  /*             BEFORE LOOP, s_new = INITIAL STATE OF THE COMPOSITION       */
  /*                                                                         */
  /* s_acc (RRR) WILL CONTAIN ALL REACHED STATES                             */

  s_new = c->initialBDD;
  s_acc = s_new;
  s1 = bdd_termFalse;

  /* SET OF WCA'S FINAL STATES*/

  finalWCA=0;
  finalBddWCA = bdd_termFalse;

  for(i=0; i < pa_processTable[c->tableP[1]].numStates; i++){
	if (pa_processTable[c->tableP[1]].tableS[i].final){
		finalWCA++;
		finalBddWCA = Bdd_ITE(finalBddWCA,bdd_termTrue,pa_processTable[c->tableP[1]].tableS[i].bddR);
	}
  }

  printf("\nfinalBddWCA:\n");
  Bdd_WriteFunction(finalBddWCA);

  /* LOOP UNTIL NO NEW STATE IS REACHED */

  tr1 = bdd_termFalse;
  tr2 = bdd_termFalse;
  do {  

    /* THIS PART IS FOR DEBUGGING */
    /**/
    printf("\n*****************************\n");
    printf("STARTING ITERATION %d",++iteration);
    printf("\n*****************************\n");
    /**/

    /* PART ONE */
    /* CALCULATE NEW STATES, WHICH ARE REACHED WITHOUT COMMUNICATION */
    /* BETWEEN PROCESSES AND STORE THEM IN new1 (RRR) */
    /* TRANSITIONS GO TO tr1 (RRRASSS) */ 

    new1 = bdd_termFalse;

    /* COMPUTE ALL TRANSITIONS IN PROCESSES lts AND wca WHICH ARE ALLOWED */  
    /* WITHOUT COMMUNICATION AND STORE THEM IN s1 (RAS) */

    s1 = Bdd_ITE(actionsBDD,pa_processTable[c->tableP[0]].d,bdd_termFalse);
    s2 = Bdd_ITE(actionsBDD,pa_processTable[c->tableP[1]].d,bdd_termFalse);

    /* UPDATE s1 AND s2 */                                          
    /* THEY SHOULD CONTAIN ONLY TRANSITIONS STARTING FROM s_new (RRR) */
    /* s1 AND s2 BECAME (RRRAS) */

    s1 = Bdd_ITE(s1,s_new,bdd_termFalse);
    /* s1 je mogoce prevelik */
    /* moramo brisati tiste prehode, ki startajo v koncnem stanju */
    s1 = Bdd_ITE(s1,Bdd_NOT(finalBddWCA),bdd_termFalse);

    s2 = Bdd_ITE(s2,s_new,bdd_termFalse);

    tr1 = Bdd_ITE(tr1,bdd_termTrue,Bdd_ITE(s1,pa_processTable[c->tableP[1]].stab,bdd_termFalse));
    tr1 = Bdd_ITE(tr1,bdd_termTrue,Bdd_ITE(s2,pa_processTable[c->tableP[0]].stab,bdd_termFalse));

    /* COMPUTE STATES IN c REACHED WITH TRANSITIONS FROM */
    /* s1 AND s2, RESPECTIVELY, AND STORE THEM IN s1_next (RRR) AND s2_next (RRR)*/
    /* s1_next AND s2_next ARE COMPLETE, THUS ADDING stab IS NOT NEEDED */

    s1_next = Bdd_NewState1(s1,pa_processTable[c->tableP[0]].name);
    s2_next = Bdd_NewState1(s2,pa_processTable[c->tableP[1]].name);

    /* ADD STATES FROM s1_next AND s2_next TO new1 (RRR) */

    new1 = Bdd_ITE(new1,bdd_termTrue,s1_next);
    new1 = Bdd_ITE(new1,bdd_termTrue,s2_next);

    /* THIS PART IS FOR DEBUGGING */
    /**/
    printf("\nBDD for new1:\n");
    Bdd_WriteFunction(new1);
    Pa_DecodeCompStates(c,new1,TRUE);
    printf("\n");
    /**/

    /* PART TWO */
    /* CALCULATE NEW STATES, WHICH ARE REACHED WITH COMMUNICATION */
    /* BETWEEN PROCESSES AND STORE THEM IN new2 (RRR) */
    /* TRANSITIONS GO TO tr2 (RRRASSS) */ 

    new2 = bdd_termFalse;

    /* COMPUTE TRANSITIONS WITHOUT a0 */
    /* s1 (RAS) = IN PROCESS lts WITH OUTPUT ACTIONS */
    /* s2 (RAS) = IN PROCESS wca WITH INPUT ACTIONS */

    s1 = Bdd_Restrict(pa_processTable[c->tableP[0]].d,
                      pa_sortTable[c->sort].a0.p->v, TRUE);

    s2 = Bdd_Restrict(pa_processTable[c->tableP[1]].d,
                      pa_sortTable[c->sort].a0.p->v, FALSE);

    /* COMPOSE s1 (RAS) AND s2 (RAS) TOGETHER AND CREATE s3 (RRRSSS) */

    s3 = Bdd_RelOp(s1,s2,"#AND Ex xA",TRUE);

    /* COMPUTE TRANSITIONS WITHOUT a0 */
    /* s1 (RAS) = IN PROCESS lts WITH INPUT ACTIONS  */
    /* s2 (RAS) = IN PROCESS wca WITH OUTPUT ACTIONS */

    s1 = Bdd_Restrict(pa_processTable[c->tableP[0]].d,
                      pa_sortTable[c->sort].a0.p->v, FALSE);

    s2 = Bdd_Restrict(pa_processTable[c->tableP[1]].d,
                      pa_sortTable[c->sort].a0.p->v, TRUE);

    /* COMPOSE s1 (RAS) AND s2 (RAS) TOGETHER AND ADD TO s3 (RRRSSS) */

    s3 =  Bdd_ITE(s3,bdd_termTrue,Bdd_RelOp(s1,s2,"#AND Ex xA",TRUE));

    /* UPDATE s3 */
    /* IT SHOULD CONTAIN ONLY TRANSITIONS STARTING FROM s_new (RRR) */
 
    s3 = Bdd_ITE(s3,s_new,bdd_termFalse);

    /* ADD TRANSITIONS TO tr2 */

    tr2 = Bdd_ITE(tr2,bdd_termTrue,s3);

    /* COMPUTE STATES IN c REACHED WITH TRANSITIONS FROM */
    /* s3 AND STORE THEM IN s3_next (RRR) */
    /* s3_next IS COMPLETE, THUS ADDING stab IS NOT NEEDED */

    s3_next = Bdd_NewState2(s3,
                            pa_processTable[c->tableP[0]].name,
                            pa_processTable[c->tableP[1]].name);

    /* ADD STATES FROM s3_next TO new2 (RRR) */

    new2 = Bdd_ITE(new2,bdd_termTrue,s3_next);

    /* THIS PART IS FOR DEBUGGING */
    /**/
    printf("\nBDD for new2:\n");
    Bdd_WriteFunction(new2);
    Pa_DecodeCompStates(c,new2,TRUE);
    printf("\n");
    /**/
    
    /* UPDATE s_new (RRR) */
    s_new = Bdd_ITE(new1,bdd_termTrue,new2);
    s_new = Bdd_ITE(s_acc,bdd_termFalse,s_new);

    /* UPDATE s_acc (RRR) */
    s_acc = Bdd_ITE(s_acc,bdd_termTrue,s_new);

  } while(!(Bdd_Cmp(s_new,bdd_termFalse)));

  /* s_acc ARE ALL STATES IN COMPOSITION c */
  c->stateBDD = s_acc;

  /* CREATE COMPLETE TRANSITION RELATION FOR c */

  tr2 = Bdd_ITE(tau,tr2,bdd_termFalse); /* tr2 = 'tau' * tr2 */ 
  c->transitionBDD = Bdd_ITE(tr1,bdd_termTrue,tr2);

  /* THIS PART IS FOR DEBUGGING */
  /**/
  printf("\nCOMPOSITION IS CREATED\n");
  Pa_DecodeComposition(name,TRUE);
  printf("\nEND OF COMPOSITION CREATION\n");
  /**/

  /* PERFORM MINIMIZATION */

  Strucval_FinalStates(c);

}


/**Function****************************************************************
  Synopsis    [void Strucval_FinalStates(Pa_Composition *c)]
  Description [calculates final states of the composition starting from 
				bottom to to top]
  SideEffects []
  SeeAlso     []
  ************************************************************************/
void
Strucval_FinalStates(Pa_Composition *c)
{
	int i,finalWCA,k,wca;
	Bdd_Edge finalBddWCA, delta_acc;
	Bdd_Edge finalSynPStatesR,finalSynPStatesS,finalSynPStates;
	Bdd_Edge newFinalSynP;

	finalWCA = 0;
	delta_acc = bdd_termFalse;
	finalBddWCA = bdd_termFalse;
	finalSynPStates = bdd_termFalse;
	finalSynPStatesR = bdd_termFalse;
	finalSynPStatesS = bdd_termFalse;
	newFinalSynP = bdd_termFalse;

	wca = Pa_FindProcess("WCA");
	/* printf("\n pos: %d", wca); */

	for(i=0; i < pa_processTable[c->tableP[wca]].numStates; i++){
		if (pa_processTable[c->tableP[wca]].tableS[i].final){
			finalWCA++;
			finalBddWCA = Bdd_ITE(finalBddWCA,bdd_termTrue,pa_processTable[c->tableP[wca]].tableS[i].bddR);
		}
	}

	/*
        printf("\n wca broj: %d ", finalWCA);
	printf("\n finalBddWCA: \n");
	Bdd_WriteFunction(finalBddWCA);

	printf("\n stateBDD: \n");
	Bdd_WriteFunction(c->stateBDD);
	*/

	/* FINAL STATES OF THE SYNCHRONOUS PRODUCT (RRR): all states of SynProd x final WCA states */
	finalSynPStatesR = Bdd_ITE(finalBddWCA,c->stateBDD,bdd_termFalse); /*AND*/

	/*
	printf("\n finalSynPStatesR: \n");
	Bdd_WriteFunction(finalSynPStatesR);
	*/

	/* ACCUMULATE FINAL STATES */
	finalSynPStates = Bdd_ITE(finalSynPStates,bdd_termTrue,finalSynPStatesR);

	/* RENAME FINAL STATES FROM (RRR) TO (SSS) */
	finalSynPStatesS = Bdd_R2S(finalSynPStatesR);

	/*
	printf("\n finalSynPStatesS: \n");
	Bdd_WriteFunction(finalSynPStatesS);
	*/

	delta_acc = Bdd_ITE(finalSynPStatesS,c->transitionBDD,bdd_termFalse);

	/*
	printf("\n delta_acc: \n");
	Bdd_WriteFunction(delta_acc);
	*/

	newFinalSynP = Bdd_ITE(Bdd_ExS(Bdd_ExA(delta_acc, bdd_termTrue),bdd_termTrue),Bdd_NOT(finalSynPStates),bdd_termFalse);

	/*
	printf("\n newFinalSynP: \n");
	Bdd_WriteFunction(newFinalSynP);
	*/

	do{

	  /* ACCUMULATE FINAL STATES */
	  finalSynPStates = Bdd_ITE(finalSynPStates,bdd_termTrue,newFinalSynP);

	  /*
	  printf("\n finalSynPStates: \n");
	  Bdd_WriteFunction(finalSynPStates);
          Pa_DecodeCompStates(c,finalSynPStates,TRUE);
	  */

	  newFinalSynP = Bdd_R2S(newFinalSynP);

	  /*
	  printf("\n newFinalSynP: \n");
	  Bdd_WriteFunction(newFinalSynP);
	  */

	  /* CALCULATE TRANSITIONS WHICH LEAD TO FINAL STATES */
	  delta_acc = Bdd_ITE(newFinalSynP,c->transitionBDD,bdd_termFalse); /*AND*/

	  /*
	  printf("\n delta_acc: \n");
	  Bdd_WriteFunction(delta_acc);
	  */

	  /* REMOVE (AAA) AND (SSS) FROM TRANSITION RELATION x NOT finalSynPStates (EXCLUDE REPEATED STATES) */
	  newFinalSynP = Bdd_ITE(Bdd_ExS(Bdd_ExA(delta_acc, bdd_termTrue),bdd_termTrue),Bdd_NOT(finalSynPStates),bdd_termFalse);
	  /*
	  printf("\n newFinalSynP: \n");
	  Bdd_WriteFunction(newFinalSynP);
	  */	

	} while(!(Bdd_Cmp(newFinalSynP,bdd_termFalse)));

	printf("\n c->stateBDD: \n");
	Bdd_WriteFunction(c->stateBDD);

	printf("\n finalSynPStates: \n");
        Bdd_WriteFunction(finalSynPStates);
	Pa_DecodeCompStates(c,finalSynPStates,TRUE);
	printf("\n");

}

/**Function****************************************************************
  Synopsis    [static int wordNumber(Bdd_String s)]
  Description [translates string s to number]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
wordNumber(Bdd_String s)
{
  int num;
  int token;
  Bdd_String sup,sup_next;

  num = 0;
  sup = strdup(s);

  token = strspn(sup," ");
  while (strspn(sup," ") < strlen(sup)) {
    sup_next = (Bdd_String) strdup(&sup[token]);
    free(sup);
    sup = sup_next;

    num++;
    token = strcspn(sup," ");
    sup_next = (Bdd_String) strdup(&sup[token]);
    free(sup);
    sup = sup_next;

    token = strspn(sup," ");
  }

  free(sup);
  return num;
}

