/**CFile*******************************************************************
  PackageName [versis]
  FileName    [versis.h]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]

  Revision    [$Revision: 2.11 $]
  Date        [$Date: 2005/08/03 07:10:53 $]
  Synopsis    [Package 'versis' provides tools for formal verification
               of concurrent systems]
  Description [The file versis.h contains declaration of all external
               data structures neded for formal verification
               of concurrent systems.]
  SeeAlso     [versisInt.h, versisMain.c ...]
  Copyright   [Copyright (C) 2000-2005 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#ifndef _VERSIS
#define _VERSIS

#define VERSIS_RELEASE "2.3"

#include <pthread.h>
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#include <unistd.h>
#include <math.h>

#include <bdd.h>
#include <pa.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */

#ifdef ESTUNIX
#  ifndef EXTERN
#  define EXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#ifndef TCL
#    ifdef BUILD_VERSIS
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#    else
#      ifdef USE_VERSIS
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#      else
#        define EXTERN extern
#      endif
#    endif
#else
#    ifdef BUILD_VERSIS
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLEXPORT
#    else
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLIMPORT
#    endif
#endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

#ifndef _COMMONTYPE
#define _COMMONTYPE
typedef void *pvoid;
#endif

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

EXTERN int versis_status;

/*-----------------------------------------------------------------------*/
/* Macro declarations                                                    */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

EXTERN void Versis_InitPkg();

EXTERN void Versis_ExitPkg();

EXTERN void Versis_AboutPkg();

EXTERN void Versis_Compose(Bdd_String name, int type,
                           Bdd_String process, Bdd_String actions);

EXTERN Bdd_Boolean Versis_Equivalence(int type, int par1, Bdd_String name1,
                                      int par2, Bdd_String name2);

EXTERN int Versis_Minimization(int type, int par, Bdd_String name);

EXTERN Bdd_Boolean Versis_Equivalence(int type, int par1, Bdd_String name1,
                                      int par2, Bdd_String name2);

EXTERN void Versis_ChangeRename(Bdd_String process, Bdd_String name1,
                                Bdd_String name2);

EXTERN void Versis_ChangeHide(Bdd_String process, Bdd_String name1);

EXTERN void Versis_ChangeForbid(Bdd_String process, Bdd_String name1);

EXTERN Bdd_Edge Versis_CheckDeadlock(Bdd_String process);

EXTERN Bdd_Edge Versis_CheckDivergence(Bdd_String process);

EXTERN int Versis_ParseCCS(Bdd_String f);

/**AutomaticEnd***********************************************************/

#endif /* _VERSIS */

