/**CFile*******************************************************************
  PackageName [versis]
  FileName    [versisCCS.y]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]

  Revision    [$Revision: 2.11 $]
  Date        [$Date: 2005/01/25 10:14:59 $]
  Synopsis    [Package 'versis' provides tools for formal verification
               of concurrent systems]
  Description [The file versisCCS.y contains parser for CCS]
  SeeAlso     [versis.h, versisInt.h]
  Copyright   [Copyright (C) 2000-2003 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

%{
#include "versisInt.h"
#define YYERROR_VERBOSE 
%}
	
%{
   static void outline(char *s);
   static void lowercase(char *s);
   static void addToSet(int **set, int *len, int state);
   static void writeSet(int *set, int len);
%}

%{
   /* UNEXPECTED SIDEEFFECT ON SOME SYSTEMS */
   /* THE VARIABLES DEFINED HERE ARE NOT ALWAYS LOCAL, THEY CAN BE GLOBAL! */
   /* THEREFORE, I USE A PREFIX FOR EACH VARIABLE */

   /* STATIC VARIABLES? I DO NOT KNOW, IF THIS IS A GOOD OR A BAD IDEA */
   /*
   static int CCSSort;
   static int CCSProc;
   static Bdd_String CCSLowerProc;
   static Bdd_String CCSPlist;
   static Bdd_String CCSAlist;
   static Bdd_String CCSOld,CCSNew;
   static int CCSNum,CCSNump,CCSNuma;
   */

   int CCSSort;
   int CCSProc;
   Bdd_String CCSLowerProc;
   Bdd_String CCSPlist;
   Bdd_String CCSAlist;
   Bdd_String CCSOld,CCSNew;
   int CCSNum,CCSNump,CCSNuma;
%}

%union {
  Bdd_String outline;
  Bdd_String word;
  struct {
    int *set;
    int len;
  } setOfStates;
}

%token UNKNOWN
%token <outline> OUTLINE
%token <word> NAME ACTION SEP NET COMPOSE MULTICOMPOSE RENAME RESTRICT
%type <setOfStates> procbody

%left '+'
%left SEP

%%
  start		: processes ;

  processes	: proces processes 
                | ;

  proces	: OUTLINE {

                   Bdd_String tmp;

                   tmp = strdup($1);
                   outline(tmp);
                   free(tmp);
                   free($1);
                  }

                | NAME {  

                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    name = strdup($1);
                    printf("  Process %s ... ",name);

		    CCSSort = Pa_AddNewSort(name);
                    CCSProc = Pa_AddNewProcess(name);
                    pa_processTable[CCSProc].sort = CCSSort;
                    CCSLowerProc = strdup(name);
                    lowercase(CCSLowerProc);
                    free(name);
		  }
                  sort {
                  }
                  '='
                  procbody {

		    int i,j;
                    int state;
                    Bdd_String tmp;
                    Bdd_Boolean OK;

                    OK = TRUE;

 		    if ($6.len == 1) {
                      pa_processTable[CCSProc].initial = $6.set[0];
		    } else {

                      for (i=0; i<$6.len; i++) {
                        tmp = strdup(pa_processTable[CCSProc].tableS[$6.set[i]].name);
                        if (!strchr(tmp,'<')) {
                          OK = FALSE;
		        }
                        free(tmp);
		      }

                      if (!OK) {
                        printf("\n  ERROR in CCS parser: operator '+' is not fully supported, yet!\n");
		      }

                      for (i=0; i<$6.len; i++) {
                        Pa_DeleteStateProcess(&pa_processTable[CCSProc],$6.set[i]);
		      }

                      state = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);
                      for (i=0; i<$6.len; i++) {
                        for(j=0; j<pa_processTable[CCSProc].numTransitions; j++) {
                          if (pa_processTable[CCSProc].tableT[j].source == $6.set[i]) {
		            pa_processTable[CCSProc].tableT[j].source = state;
		          }
		        }
		      }
                      pa_processTable[CCSProc].initial = state;
		    }

                    if (OK) {
                      printf("OK\n");
		    }

                    free($1);
                    free($6.set);
                    free(CCSLowerProc);
                  }

                | NET NAME {  

                    Bdd_String name;

		    /*
                    fprintf(stderr,"NET NAME %s\n",$2);
		    */

                    name = strdup($2);
                    printf("  Net %s\n",name);
                    printf("    Parallel composition... ");

		    CCSSort = Pa_AddNewSort(name);

                    CCSNump = 0;
                    CCSNuma = 0;
                    CCSPlist = NULL;
                    CCSAlist = NULL;

                    free(name);
		  }
                  '='
                  compbody {

                    Bdd_String comp;

                    comp = strdup($2);

                    Pa_EncodeSort(pa_sortTable[CCSSort].name);
                    Pa_EncodeMultiProcesses(CCSPlist);
                    Versis_Compose(comp,2,CCSPlist,CCSAlist);

                    printf("OK\n");
                    printf("    Creating process %s... ",comp);
                    Pa_Composition2Process(comp,TRUE);
                    printf("OK\n");

                    free(comp);
                    free(CCSPlist);
                    free(CCSAlist);
                    free($2);
                  };

  sort          : '(' sortlist ')'
                | ;


  sortlist      : NAME {
                    Bdd_String name;

                    name = strdup($1);
                    Pa_FOASortAction(&pa_sortTable[CCSSort],name);
                    free(name);
                    free($1);
                  }
                  sortlist

		| ;

  procbody      : NAME {

                    int state;
                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME == %s\n",$1);
		    */

                    name = strdup($1);
                    state = Pa_FOAGateProcess(&pa_processTable[CCSProc],name);

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state);

                    free(name);
                    free($1);
		  }

                | procbody '+' procbody {

		    int i;

		    /*
                    fprintf(stderr,"PLUS\n");
		    */

                    $$.set = NULL;
                    $$.len = 0;
                    for (i=0; i<$1.len; i++) {
                      addToSet(&$$.set,&$$.len,($1.set)[i]);
		    }
                    for (i=0; i<$3.len; i++) {
                      addToSet(&$$.set,&$$.len,($3.set)[i]);
		    }

                    free($1.set);
                    free($3.set);
                  }

                | '(' procbody ')' {
                    $$.set = $2.set;
                    $$.len = $2.len;
                  }

                | ACTION SEP procbody {

		    int i;
		    int state,len,act,type;
                    Bdd_String action;

		    /*
                    fprintf(stderr,"ACTION %s\n",$1);
                    */

                    type = 0;
                    action = strdup($1);
                    state = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],action);
		    }

                    for (i=0; i<$3.len; i++) {
                      Pa_FOATransition(&pa_processTable[CCSProc], 
                                        state, act, type, ($3.set)[i]);
                    }

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state);

		    /*
                    writeSet($$.set,$$.len);
         	    */

                    free($3.set);
                    free(action);
                    free($1);
		  } ;

  compbody      : COMPOSE '(' proclist ')' restrlist {

                    /*
		    fprintf(stderr,"COMPOSE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
		    fprintf(stderr,"(numa = %d) %s\n",CCSNuma,CCSAlist);
		    */

		  }

                | MULTICOMPOSE '(' proclist ')' restrlist {
		    printf("WARNING: LOTOS composition is not allowed, yet!\n");
		  } ;

  proclist	: oneproc ',' proclist 
                | oneproc ;

  oneproc       : NAME {

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    CCSNum = 0;
                    CCSOld = NULL;
                    CCSNew = NULL;

                  } renlist {

                    Bdd_String name,newproc;
                    Bdd_String tmp;
                    int i;

                    name = strdup($1);

                    i = 0;
                    newproc = NULL;
                    do {
                      i++;
                      if (newproc) free(newproc);
                      newproc = (Bdd_String) malloc(127);
                      sprintf(newproc,"%s%d",name,i);
                    } while (Pa_FindProcess(newproc) != -1);

                    CCSNump++;
                    if (!CCSPlist) {
                      CCSPlist = strdup(newproc);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSPlist,newproc);
                      free(CCSPlist);
                      CCSPlist = strdup(tmp);
                      free(tmp);
		    }

                    /*
                    fprintf(stderr,"Renaming old = [%s]\n",CCSOld);
                    fprintf(stderr,"Renaming new = [%s]\n",CCSNew);
		    */

                    Pa_CopyProcess(name,newproc,pa_sortTable[CCSSort].name,CCSOld,CCSNew);

                    free(name);
                    free(newproc);
                    free(CCSOld);
                    free(CCSNew);
                    free($1);
                  };

  renlist       : renaction renlist
                | ;

  renaction     : '['
                  NAME
                  RENAME
                  NAME
                  ']' {

                    Bdd_String name1,name2;
                    Bdd_String tmp;

                    name1 = strdup($2);
                    name2 = strdup($4);

                    CCSNum++;

                    Pa_FOASortAction(&pa_sortTable[CCSSort],name1);
                    if (!CCSNew) {
                      CCSNew = strdup(name1);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSNew,name1);
                      free(CCSNew);
                      CCSNew = strdup(tmp);
                      free(tmp);
		    }

                    Pa_FOASortAction(&pa_sortTable[CCSSort],name2);
                    if (!CCSOld) {
                      CCSOld = strdup(name2);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSOld,name2);
                      free(CCSOld);
                      CCSOld = strdup(tmp);
                      free(tmp);
		    }

                    free(name1);
                    free(name2);

                    free($2);
                    free($4);
                  } ;

  restrlist     : restraction restrlist
                | ;

  restraction   : RESTRICT NAME {

                    Bdd_String name;
                    Bdd_String tmp;

                    /*
                    fprintf(stderr,"NAME %s\n",$2);
		    */

                    name = strdup($2);

                    Pa_FOASortAction(&pa_sortTable[CCSSort],name);

                    CCSNuma++;
                    if (!CCSAlist) {
                      CCSAlist = strdup(name);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSAlist,name);
                      free(CCSAlist);
                      CCSAlist = strdup(tmp);
                      free(tmp);
		    }

                    free(name);
                    free($2);
                  };

%%

#include <lex.ccs.c>

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Versis_ParseCCS(Bdd_String f)
{
  printf("\n");
  yyin = fopen(f,"r");
  if (yyin != NULL) {
    yyparse();
    fclose(yyin);
  } else {
    printf("\n  ERROR: I can not open file %s.\n",f);
  }

  return 0;
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
outline(char *s)
{
  if (strcmp(s,"")) printf(&s[1]);
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
lowercase(char *s)
{
   int  i = 0;

   while ( s[i] )
   {
      s[i] = tolower(s[i]);
      i++;
   }

   return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToSet(int **set, int *len, int state)
{
  if (!*set) {
    *set = (int *) malloc (sizeof(int));
    (*set)[0] = state;
    *len = 1;
  } else {
    (*len)++;
    *set = (int *) realloc (*set, (*len) * sizeof(int));
    (*set)[(*len) - 1] = state;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
writeSet(int *set, int len)
{
  int i;

  fprintf(stderr,"SET:");
  for (i=0; i<len; i++) {
    fprintf(stderr," %d",set[i]);
  }
  fprintf(stderr,"\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of parser internal functions                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yyerror(char *s)
{
  printf("\n  ERROR: ");
  if (strstr(s,"parse error, unexpected UNKNOWN")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (strcmp(s,"parse error, unexpected UNKNOWN, expecting ACTION")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected NAME, expecting ACTION")) {
     printf("incorrect action name '%s', missing ! or ?",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected ACTION, expecting NAME")) {
     printf("incorrect process name '%s'",yytext);
  } else {
    printf("%s",s); 
  }
  printf("\n");
  return 1;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yywrap()
{
  return 1;
}
