/**CFile*******************************************************************
  PackageName [versis]
  FileName    [versisInt.h]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]

  Revision    [$Revision: 2.6 $]
  Date        [$Date: 2005/01/03 14:54:03 $]
  Synopsis    [Package 'versis' provides tools for formal verification
               of concurrent systems]
  Description [The file versisInt.h contains declaration of internal
               data structures neded for formal verification
               of concurrent systems.]
  SeeAlso     [versis.h, versisMain.c ...]
  Copyright   [Copyright (C) 2000-2004 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#ifndef _VERSISINT
#define _VERSISINT

#ifndef TCLOUT
#  define Mi_Out printf
   char outthread;
   int outflag;
   char *out;
#else
#  include <mi.h>
#  define printf Mi_Printf
#endif

#include "versis.h"

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

typedef int *TTabIndexov;   /* tabela indeksov spremenljivk */

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

typedef struct {
  Bdd_Edge S;
  Bdd_Boolean b;
  int t;
} VersisST;

typedef struct {
  int num;
  VersisST *table;
} VersisStateTable;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

extern Bdd_Boolean VersisStrongEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2);

extern Bdd_Boolean VersisWeakEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2);

extern Bdd_Boolean VersisTestEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2,
                                      int *acc1, int *acc2);

extern Bdd_Boolean VersisTraceEquivalence(int type, Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2,
                                      int *det1, int *det2);

extern int VersisStrongMinimization(Bdd_String name, int par, int n);

extern int VersisWeakMinimization(Bdd_String name, int par, int n);

extern int VersisTraceMinimization(int type, Bdd_String name, int par, int n);

extern int VersisWCAMinimization(Bdd_String name, int par, int n);

extern Bdd_Edge VersisBuildDet(Bdd_String orgname, Bdd_String newname,
			Bdd_Edge init, Bdd_Edge Ptau, Bdd_Edge trcl, int s);

extern Bdd_Edge VersisTrCl(Bdd_Edge f);

extern Bdd_Edge VersisComposeActions(Pa_Composition *c);

/**AutomaticEnd***********************************************************/

#endif /* _VERSISINT */











