/**CFile*******************************************************************
  PackageName [versis]
  FileName    [versisThread.c]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]

  Revision    [$Revision: 2.2 $]
  Date        [$Date: 2004/07/22 14:33:08 $]
  Synopsis    [Package 'versis' provides tools for formal verification
               of concurrent systems]
  Description [The file versisThread.c contains functions needed if using
               threads]
  SeeAlso     [versis.h, versisInt.h]
  Copyright   [Copyright (C) 2000-2004 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#include "versisInt.h"

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of thread functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
VersisCompose_Thr(pvoid args)
{
  struct funparam {
    Bdd_String name;
    int type;
    Bdd_String process;
    Bdd_String actions;
    int *thread_done;
  };

  Bdd_String name;
  int type;
  Bdd_String process;
  Bdd_String actions;

  name = ((struct funparam *) args)->name;
  type = ((struct funparam *) args)->type;
  process = ((struct funparam *) args)->process;
  actions = ((struct funparam *) args)->actions;

  Versis_Compose(name,type,process,actions);

  *((struct funparam *) args)->thread_done = 1;

  return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
VersisEquivalence_Thr(pvoid args)
{
  struct funparam {
    int type;
    int par1;
    Bdd_String name1;
    int par2;
    Bdd_String name2;
    Bdd_Boolean *r;
    int *thread_done;
  };

  int type;
  int par1;
  Bdd_String name1;
  int par2;
  Bdd_String name2;
  Bdd_Boolean r;

  type = ((struct funparam *) args)->type;
  par1 = ((struct funparam *) args)->par1;
  name1 = ((struct funparam *) args)->name1;
  par2 = ((struct funparam *) args)->par2;
  name2 = ((struct funparam *) args)->name2;

  r = Versis_Equivalence(type,par1,name1,par2,name2);

  *((struct funparam *) args)->r = r;
  *((struct funparam *) args)->thread_done = 1;

  return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
VersisMinimization_Thr(pvoid args)
{
  struct funparam {
    int type;
    int par;
    Bdd_String name;
    int *thread_done;
  };

  int type;
  int par;
  Bdd_String name;

  type = ((struct funparam *) args)->type;
  par = ((struct funparam *) args)->par;
  name = ((struct funparam *) args)->name;

  Versis_Minimization(type,par,name);

  *((struct funparam *) args)->thread_done = 1;

  return;
}

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_Compose_Thr(Bdd_String name, int type, int numP, Bdd_String process,
                   int numA, Bdd_String actions)
{
  struct {
    Bdd_String name;
    int type;
    Bdd_String process;
    Bdd_String actions;
    int *thread_done;
  } funparam;

  pthread_t thread;
  pthread_attr_t attr;
  struct sched_param param;

  int thread_done;

  /*
  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr,262144);
  pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy(&attr,SCHED_OTHER);
  param.sched_priority = sched_get_priority_min(SCHED_OTHER);
  pthread_attr_setschedparam(&attr, &param);
  */

  funparam.name = name;
  funparam.type = type;
  funparam.process = process;
  funparam.actions = actions;
  funparam.thread_done = &thread_done;

  /* POSIX */
  outflag = 0;
  outthread = TRUE;
  thread_done = 0;
  if (pthread_create(&thread,NULL,(void *)&VersisCompose_Thr,&funparam)) {
    outthread = FALSE;
    printf("Error: pthread_create\n");
  } else {
    sched_yield();
    while (!thread_done) {
#ifdef TCLOUT
      Tcl_DoOneEvent(TCL_DONT_WAIT);
#endif
      if (outflag) {
        Mi_Out("%s",out);
        outflag = 0;
      }
    }
    if (pthread_join(thread,NULL)) {
      Mi_Out("Error: pthread_join\n");
    }
    outthread = FALSE;
  }

  return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Boolean
Versis_Equivalence_Thr(int type, int par1, Bdd_String name1,
                       int par2, Bdd_String name2)
{
  struct {
    int type;
    int par1;
    Bdd_String name1;
    int par2;
    Bdd_String name2;
    Bdd_Boolean *r;
    int *thread_done;
  } funparam;

  pthread_t thread;
  pthread_attr_t attr;
  struct sched_param param;

  Bdd_Boolean r;
  int thread_done;

  /*
  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr,262144);
  pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy(&attr,SCHED_OTHER);
  param.sched_priority = sched_get_priority_min(SCHED_OTHER);
  pthread_attr_setschedparam(&attr, &param);
  */

  funparam.type = type;
  funparam.par1 = par1;
  funparam.name1 = name1;
  funparam.par2 = par2;
  funparam.name2 = name2;
  funparam.r = &r;
  funparam.thread_done = &thread_done;

  /* POSIX */
  outflag = 0;
  outthread = TRUE;
  thread_done = 0;
  if (pthread_create(&thread,NULL,(void *)&Versis_Equivalence_Thr,&funparam)) {
    outthread = FALSE;
    printf("Error: pthread_create\n");
  } else {
    sched_yield();
    while (!thread_done) {
#ifdef TCLOUT
      Tcl_DoOneEvent(TCL_DONT_WAIT);
#endif
      if (outflag) {
        Mi_Out("%s",out);
        outflag = 0;
      }
    }
    if (pthread_join(thread,NULL)) {
      Mi_Out("Error: pthread_join\n");
    }
    outthread = FALSE;
  }

  return r;
}



/**Function****************************************************************
  Synopsis    [Function Versis_Minimization.]
  Description [type = 0,1,2 - Strong, Weak, Trace]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_Minimization_Thr(int type, int par, Bdd_String name)
{
  struct {
    int type;
    int par;
    Bdd_String name;
    int *thread_done;
  } funparam;

  pthread_t thread;
  pthread_attr_t attr;
  struct sched_param param;

  int thread_done;

  /*
  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr,262144);
  pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy(&attr,SCHED_OTHER);
  param.sched_priority = sched_get_priority_min(SCHED_OTHER);
  pthread_attr_setschedparam(&attr, &param);
  */

  funparam.type = type;
  funparam.par = par;
  funparam.name = name;
  funparam.thread_done = &thread_done;

  /* POSIX */
  outflag = 0;
  outthread = TRUE;
  thread_done = 0;
  if (pthread_create(&thread,NULL,(void *)&VersisMinimization_Thr,&funparam)) {
    outthread = FALSE;
    printf("Error: pthread_create\n");
  } else {
    sched_yield();
    while (!thread_done) {
#ifdef TCLOUT
      Tcl_DoOneEvent(TCL_DONT_WAIT);
#endif
      if (outflag) {
        Mi_Out("%s",out);
        outflag = 0;
      }
    }
    if (pthread_join(thread,NULL)) {
      Mi_Out("Error: pthread_join\n");
    }
    outthread = FALSE;
  }

  return;
}
