/**CFile*******************************************************************
  PackageName [mi]
  FileName    [mi.c]
  Authors     [Robert Meolic (meolic@uni-mb.si)]

  Revision    [$Revision$]
  Date        [$Date$]
  Synopsis    []
  Description []
  SeeAlso     [mi.h]
  Copyright   [Copyright (C) 2000-2004 Robert Meolic
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#include "mi.h"

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
#define USECONST const

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

char outthread;
int outflag;
char *out;
Tcl_Interp *outinterp;

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int MiAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);


/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Mi_Init(Tcl_Interp *interp)
{
  outinterp = interp;
  outthread = 0;

  Tcl_CreateCommand(interp, "mi_aboutPkg", MiAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  return Tcl_PkgProvide(outinterp, "mi", "1.0");
}

/**Function****************************************************************
  Synopsis    [Function Mi_AboutPkg]
  Description [Mi_AboutPkg reports release of My interface package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mi_AboutPkg() {
  Mi_Out("My interface package, release ");
  Mi_Out(MI_RELEASE);
  Mi_Out("\n");
}

/**Function****************************************************************
  Synopsis    [Function miPrintf.]
  Description [This function redirect printf to GUI.]
  SideEffects [To write text to stdout please use fprintf(stdout,...)]
  SeeAlso     []
  ************************************************************************/

int
Mi_Printf(char *format, ...)
{
  int result;
  va_list args;
  char *cmd, *cmdnew;
  int i,j,len;

  if (outthread) {

    cmd = (char *) Tcl_Alloc(1024); /* ONLY 1024 BYTES ARE ALLOCATED !!! */

    va_start(args, format);
    vsprintf(cmd, format, args);
    va_end(args);

    out = cmd;
    outflag = 1;
    while (outflag);
    Tcl_Free(cmd);

  } else {

    cmd = (char *) Tcl_Alloc(1024); /* ONLY 1024 BYTES ARE ALLOCATED !!! */
    cmd[0] = 'o';
    cmd[1] = 'u';
    cmd[2] = 't';
    cmd[3] = ' ';
    cmd[4] = '\"';
  
    va_start(args, format);
    vsprintf(&cmd[5], format, args);
    va_end(args);

    len = strlen(cmd);
    cmd[len] = '\"';
    cmd[len+1] = 0;
    len = strlen(cmd);

    cmdnew = (char *) Tcl_Alloc(2*len);
    i = 0;
    j = 0;
    while (i <= len) {
      if ((cmd[i] != '[') && (cmd[i] != ']')) {
        cmdnew[j++] = cmd[i++];
      } else {
        cmdnew[j++] = '\\';
        cmdnew[j++] = cmd[i++];
      }
    }

    result = Tcl_EvalEx(outinterp,cmdnew,-1,TCL_EVAL_GLOBAL);
    Tcl_Free(cmd);
    Tcl_Free(cmdnew);
    return result;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description [Without threads]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Mi_Out(char *format, ...)
{
  int result;
  va_list args;
  char *cmd, *cmdnew;
  int i,j,len;

  cmd = (char *) Tcl_Alloc(1024); /* ONLY 1024 BYTES ARE ALLOCATED !!! */
  cmd[0] = 'o';
  cmd[1] = 'u';
  cmd[2] = 't';
  cmd[3] = ' ';
  cmd[4] = '\"';
  
  va_start(args, format);
  vsprintf(&cmd[5], format, args);
  va_end(args);

  len = strlen(cmd);
  cmd[len] = '\"';
  cmd[len+1] = 0;
  len = strlen(cmd);

  cmdnew = (char *) Tcl_Alloc(2*len);
  i = 0;
  j = 0;
  while (i <= len) {
    if ((cmd[i] != '[') && (cmd[i] != ']')) {
      cmdnew[j++] = cmd[i++];
    } else {
      cmdnew[j++] = '\\';
      cmdnew[j++] = cmd[i++];
    }
  }

  result = Tcl_EvalEx(outinterp,cmdnew,-1,TCL_EVAL_GLOBAL);
  Tcl_Free(cmd);
  Tcl_Free(cmdnew);
  return result;
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function MiAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
MiAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
              USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  Mi_AboutPkg();

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}
